package sun.rmi.runtime;

import java.rmi.server.LogStream;
import java.io.ByteArrayOutputStream;
import java.util.logging.LogRecord;
import java.util.logging.SimpleFormatter;
import java.util.logging.StreamHandler;
import java.security.PrivilegedAction;
import java.util.logging.Handler;
import java.util.logging.Logger;
import java.security.AccessController;
import sun.security.action.GetPropertyAction;
import java.io.PrintStream;
import java.io.OutputStream;
import java.util.logging.Level;

public abstract class Log
{
    public static final Level BRIEF;
    public static final Level VERBOSE;
    private static final Log.LogFactory logFactory;
    
    public Log() {
    }
    
    public abstract boolean isLoggable(Level p0);
    
    public abstract void log(Level p0, String p1);
    
    public abstract void log(Level p0, String p1, Throwable p2);
    
    public abstract void setOutputStream(OutputStream p0);
    
    public abstract PrintStream getPrintStream();
    
    public static Log getLog(String s, String s2, int n) {
        Level level;
        if (n < 0) {
            level = null;
        }
        else if (n == 0) {
            level = Level.OFF;
        }
        else if (n > 0 && n <= 10) {
            level = Log.BRIEF;
        }
        else if (n > 10 && n <= 20) {
            level = Log.VERBOSE;
        }
        else {
            level = Level.FINEST;
        }
        return Log.logFactory.createLog(s, s2, level);
    }
    
    public static Log getLog(String s, String s2, boolean b) {
        return Log.logFactory.createLog(s, s2, b ? Log.VERBOSE : null);
    }
    
    private static String[] getSource() {
        StackTraceElement[] stackTrace = new Exception().getStackTrace();
        return new String[] { stackTrace[3].getClassName(), stackTrace[3].getMethodName() };
    }
    
    static String[] access$200() {
        return getSource();
    }
    
    static {
        BRIEF = Level.FINE;
        VERBOSE = Level.FINER;
        logFactory = (((boolean)Boolean.valueOf((String)AccessController.doPrivileged(new GetPropertyAction("sun.rmi.log.useOld")))) ? new Log.LogStreamLogFactory() : new Log.LoggerLogFactory());
    }
    
    private static class LoggerLogFactory implements Log.LogFactory
    {
        LoggerLogFactory() {
        }
        
        @Override
        public Log createLog(String name, String s, Level level) {
            return new Log.LoggerLog(Logger.getLogger(name), level);
        }
    }
    
    private static class LoggerLog extends Log
    {
        private static final Handler alternateConsole;
        private Log.InternalStreamHandler copyHandler;
        private final Logger logger;
        private Log.LoggerPrintStream loggerSandwich;
        
        private LoggerLog(Logger logger, Level level) {
            this.copyHandler = null;
            this.logger = logger;
            if (level != null) {
                AccessController.doPrivileged(new PrivilegedAction<Void>() {
                    @Override
                    public Void run() {
                        if (!logger.isLoggable(level)) {
                            logger.setLevel(level);
                        }
                        logger.addHandler(Log.LoggerLog.alternateConsole);
                        return null;
                    }
                });
            }
        }
        
        @Override
        public boolean isLoggable(Level level) {
            return this.logger.isLoggable(level);
        }
        
        @Override
        public void log(Level level, String str) {
            if (this.isLoggable(level)) {
                String[] access$200 = Log.access$200();
                this.logger.logp(level, access$200[0], access$200[1], Thread.currentThread().getName() + ": " + str);
            }
        }
        
        @Override
        public void log(Level level, String str, Throwable thrown) {
            if (this.isLoggable(level)) {
                String[] access$200 = Log.access$200();
                this.logger.logp(level, access$200[0], access$200[1], Thread.currentThread().getName() + ": " + str, thrown);
            }
        }
        
        @Override
        public synchronized void setOutputStream(OutputStream outputStream) {
            if (outputStream != null) {
                if (!this.logger.isLoggable(Log.LoggerLog.VERBOSE)) {
                    this.logger.setLevel(Log.LoggerLog.VERBOSE);
                }
                (this.copyHandler = new InternalStreamHandler(outputStream)).setLevel(Log.VERBOSE);
                this.logger.addHandler(this.copyHandler);
            }
            else {
                if (this.copyHandler != null) {
                    this.logger.removeHandler(this.copyHandler);
                }
                this.copyHandler = null;
            }
        }
        
        @Override
        public synchronized PrintStream getPrintStream() {
            if (this.loggerSandwich == null) {
                this.loggerSandwich = new Log.LoggerPrintStream(this.logger);
            }
            return this.loggerSandwich;
        }
        
        static {
            alternateConsole = (Handler)AccessController.doPrivileged(new PrivilegedAction<Handler>() {
                @Override
                public Handler run() {
                    Log.InternalStreamHandler internalStreamHandler = new Log.InternalStreamHandler(System.err);
                    internalStreamHandler.setLevel(Level.ALL);
                    return internalStreamHandler;
                }
            });
        }
    }
    
    private static class InternalStreamHandler extends StreamHandler
    {
        InternalStreamHandler(OutputStream out) {
            super(out, new SimpleFormatter());
        }
        
        @Override
        public void publish(LogRecord record) {
            super.publish(record);
            this.flush();
        }
        
        @Override
        public void close() {
            this.flush();
        }
    }
    
    private static class LoggerPrintStream extends PrintStream
    {
        private final Logger logger;
        private int last;
        private final ByteArrayOutputStream bufOut;
        
        private LoggerPrintStream(Logger logger) {
            super(new ByteArrayOutputStream());
            this.last = -1;
            this.bufOut = (ByteArrayOutputStream)super.out;
            this.logger = logger;
        }
        
        @Override
        public void write(int n) {
            if (this.last == 13 && n == 10) {
                this.last = -1;
                return;
            }
            Label_0111: {
                if (n != 10 && n != 13) {
                    super.write(n);
                    break Label_0111;
                }
                try {
                    this.logger.logp(Level.INFO, "LogStream", "print", Thread.currentThread().getName() + ": " + this.bufOut.toString());
                }
                finally {
                    this.bufOut.reset();
                }
            }
            this.last = n;
            return;
        }
        
        @Override
        public void write(byte[] array, int n, int index) {
            if (index < 0) {
                throw new ArrayIndexOutOfBoundsException(index);
            }
            for (int i = 0; i < index; ++i) {
                this.write((int)array[n + i]);
            }
            return;
        }
        
        @Override
        public String toString() {
            return "RMI";
        }
    }
    
    private static class LogStreamLogFactory implements Log.LogFactory
    {
        LogStreamLogFactory() {
        }
        
        @Override
        public Log createLog(String s, String name, Level level) {
            LogStream log = null;
            if (name != null) {
                log = LogStream.log(name);
            }
            return new Log.LogStreamLog(log, level);
        }
    }
    
    private static class LogStreamLog extends Log
    {
        private final LogStream stream;
        private int levelValue;
        
        private LogStreamLog(LogStream stream, Level level) {
            this.levelValue = Level.OFF.intValue();
            if (stream != null && level != null) {
                this.levelValue = level.intValue();
            }
            this.stream = stream;
        }
        
        @Override
        public synchronized boolean isLoggable(Level level) {
            return level.intValue() >= this.levelValue;
        }
        
        @Override
        public void log(Level level, String str) {
            if (this.isLoggable(level)) {
                String[] access$200 = Log.access$200();
                this.stream.println(unqualifiedName(access$200[0]) + "." + access$200[1] + ": " + str);
            }
        }
        
        @Override
        public void log(Level level, String str, Throwable t) {
            if (this.isLoggable(level)) {
                synchronized (this.stream) {
                    String[] access$200 = Log.access$200();
                    this.stream.println(unqualifiedName(access$200[0]) + "." + access$200[1] + ": " + str);
                    t.printStackTrace(this.stream);
                }
            }
        }
        
        @Override
        public PrintStream getPrintStream() {
            return this.stream;
        }
        
        @Override
        public synchronized void setOutputStream(OutputStream outputStream) {
            if (outputStream != null) {
                if (Log.LogStreamLog.VERBOSE.intValue() < this.levelValue) {
                    this.levelValue = Log.LogStreamLog.VERBOSE.intValue();
                }
                this.stream.setOutputStream(outputStream);
            }
            else {
                this.levelValue = Level.OFF.intValue();
            }
        }
        
        private static String unqualifiedName(String s) {
            int lastIndex = s.lastIndexOf(".");
            if (lastIndex >= 0) {
                s = s.substring(lastIndex + 1);
            }
            s = s.replace('$', '.');
            return s;
        }
    }
    
    private interface LogFactory
    {
        public abstract Log createLog(String p0, String p1, Level p2);
    }
}
